--------------------------------------------------------
-- Archivo creado  - viernes-junio-08-2012   
--------------------------------------------------------

--------------------------------------------------------
--  DDL for Procedure INSERTLOG
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "INSERTLOG" (modul       IN  LOG_DB.modul%TYPE,
                    dades_ent    IN  LOG_DB.dadesin%TYPE,
                    dades_sort   IN  LOG_DB.dadesout%TYPE,
                    rtn          IN  LOG_DB.resultat%TYPE,
                    id_error     IN  LOG_DB.iderror%TYPE,
                    usuari       IN  LOG_DB.usuari%TYPE)
                
IS 

/**************************************************************************************
NOM:             insertLog
FUNCIO:          Insercio dels registres a la taula de Log
PARMETRES  IN:  modul: procediment que fa la crida (ALTAPAIS, ALTACLIENT, ETC)
                 dades_ent: dades d'entrada del procediment que crida (valors amb els que es crida el procediment
                 dades_sort: dades de sortida del procediment que crida. Si KO descripcio de l'error
                 rtn: Resultat del procediment que crida (OK/KO)
                 num_error: Nmero d'error 
                 usuari: Usuari que fa la crida del modul.
**************************************************************************************/

    BEGIN

    INSERT INTO LOG_DB VALUES (modul,
                          dades_ent,
                          dades_sort,
                          rtn,
                          id_error,
                           NLS_UPPER(TRIM(usuari)),
                          current_TIMESTAMP);
      COMMIT;

END;

/

/



--------------------------------------------------------
--  DDL for Procedure ALTACENTRALDISTR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTACENTRALDISTR" (wcodi       IN  CENTRAL_DISTR.codi%TYPE,
                           wadreca     IN  CENTRAL_DISTR.adreca%TYPE,
                           wpoblacio   IN  CENTRAL_DISTR.poblacio%TYPE,
                           wcodipostal IN  CENTRAL_DISTR.codipostal%TYPE,
                           wpais       IN  CENTRAL_DISTR.pais%TYPE,
                           wenergia    IN  CENTRAL_DISTR.energiamax%TYPE,
                           wusuari     IN  CENTRAL_DISTR.usuarialta%TYPE,
                           RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaCentralDistr
FUNCIO:          Alta de les centrals de distribucio
PARMETRES  IN:  codi: Codi de la central
                 adreca:  adrea on esta situada la central
                 poblacio: poblacio on esta situada la central
                 codipostal: codi postal on esta situada la central
                 pais: pais on esta situat situada la central
                 energiamax: maxim de energia electrica (KWh) que pot suministrar la central
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CENTRALDISTR_JA_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaCentralDistr';
    xdades_ent := 'Codi: '           || wcodi       ||
                  '. Adreca: '       || wadreca     ||
                  '. Poblacio: '     || wpoblacio   ||
                  '. Codi Postal: '  || wcodipostal ||
                  '. Pais: '         || wpais       ||
                  '. Energia '       || wenergia    ||
                  '. Usuari: '       || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wadreca IS NULL) OR (wpoblacio IS NULL)
                      OR (wcodipostal IS NULL) OR (wpais IS NULL) 
                      OR (wenergia IS NULL)  OR  (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio camp CENTRAL DISTRIBUCIO no existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CENTRAL_DISTR
    WHERE codi = NLS_UPPER(TRIM(wcodi));
 
    
   IF (wcount > 0) THEN
      RAISE CODI_CENTRALDISTR_JA_EXISTENT;
   END IF;

-- Insercio de les dades
 
    INSERT INTO CENTRAL_DISTR VALUES(NLS_UPPER(TRIM(wcodi)),
                                    NLS_UPPER(TRIM(wadreca)),
                                    NLS_UPPER(TRIM(wpoblacio)),
                                    wcodipostal,
                                    NLS_UPPER(TRIM(wpais)),
                                    wenergia,
                                    SYSDATE,
                                    NULL,
                                    NULL,
                                    NLS_UPPER(TRIM(wusuari)),
                                    NULL,
                                    NULL);

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
         WHEN  CODI_CENTRALDISTR_JA_EXISTENT THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi central distribuci ja existent';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
          
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure ALTACENTRALPROD
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTACENTRALPROD" (wcodi       IN  CENTRAL_PROD.codi%TYPE,
                          wadreca     IN  CENTRAL_PROD.adreca%TYPE,
                          wpoblacio   IN  CENTRAL_PROD.poblacio%TYPE,
                          wcodipostal IN  CENTRAL_PROD.codipostal%TYPE,
                          wpais       IN  CENTRAL_PROD.pais%TYPE,
                          wtipus      IN  CENTRAL_PROD.tipus%TYPE,
                          wnummolins  IN  CENTRAL_PROD.nummolins%TYPE,
                          wnumpanels  IN  CENTRAL_PROD.numpanels%TYPE,
                          wkgCO2      IN  CENTRAL_PROD.kgCO2%TYPE,
                          wkgrebuig   IN  CENTRAL_PROD.kgrebuig%TYPE,
                          wenergiamin IN  CENTRAL_PROD.energiamin%TYPE,
                          wusuari     IN  CENTRAL_PROD.usuarialta%TYPE,
                          RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaCentralProd
FUNCIO:          Alta de les centrals de produccio
PARMETRES  IN:  codi: Codi de la central
                 adreca:  adrea on esta situada la central
                 poblacio: poblacio on esta situada la central
                 codipostal: codi postal on esta situada la central
                 pais: pais on esta situat situada la central
                 tipus: tipus de central (nuclear 'N', termica 'T', carbo 'C', eolica 'E', solar 'S')
                 nummolins: nombre molins de vent instalats per central eolica
                 numpanels: nombre panels instalats per central solar
                 kgCO2: Kilograms d'emisions de CO2 per centra termica i carbo
                 kgrebuig: Kilograms de rebuig radioactiu generat per central nuclear
                 energiamin: energia minima necesaria per funcionar per energia nuclear
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CENTRALPROD_JA_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaCentralProd';
    xdades_ent := 'Codi: '           || wcodi       ||
                  '. Adreca: '       || wadreca     ||
                  '. Poblacio: '     || wpoblacio   ||
                  '. Codi Postal: '  || wcodipostal ||
                  '. Pais: '         || wpais       ||
                  '. Tipus: '        || wtipus      ||
                  '. No. molins: '   || wnummolins  ||
                  '. No. panels: '   || wnumpanels  ||
                  '. Kg. CO2: '      || wkgCO2      ||
                  '. Kg. rebuig: '   || wkgrebuig   ||
                  '. Energia min. '  || wenergiamin ||
                  '. Usuari: '       || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wadreca IS NULL) OR (wpoblacio IS NULL)
                      OR (wcodipostal IS NULL) OR (wpais IS NULL) 
                      OR (wtipus IS NULL)  OR  (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
   IF (wtipus = 'E') AND (wnummolins IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
   IF (wtipus = 'S') AND (wnumpanels IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
   IF ((wtipus = 'T') OR (wtipus = 'C')) AND (wkgCO2 IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
   IF (wtipus = 'N') AND ((wkgrebuig IS NULL) OR (wenergiamin IS NULL)) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio camp CENTRAL PRODUCCIO no existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CENTRAL_PROD
    WHERE codi = NLS_UPPER(TRIM(wcodi));
 
    
   IF (wcount > 0) THEN
      RAISE CODI_CENTRALPROD_JA_EXISTENT;
   END IF;

-- Insercio de les dades
 
    INSERT INTO CENTRAL_PROD VALUES(NLS_UPPER(TRIM(wcodi)),
                                    NLS_UPPER(TRIM(wadreca)),
                                    NLS_UPPER(TRIM(wpoblacio)),
                                    wcodipostal,
                                    NLS_UPPER(TRIM(wpais)),
                                    NLS_UPPER(TRIM(wtipus)),
                                    wnummolins,
                                    wnumpanels,
                                    wkgCO2,
                                    wkgrebuig,
                                    wenergiamin,
                                    SYSDATE,
                                    NULL,
                                    NULL,
                                    NLS_UPPER(TRIM(wusuari)),
                                    NULL,
                                    NULL);

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
         WHEN  CODI_CENTRALPROD_JA_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi central produci ja existent';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure ALTACENTRALPRODCENTRALDISTR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTACENTRALPRODCENTRALDISTR" (wcentralprod   IN  CENTRALPROD_CENTRALDISTR.centralprod%TYPE,
                                      wlinia         IN  CENTRALPROD_CENTRALDISTR.linia%TYPE,
                                      wcentraldistr  IN  CENTRALPROD_CENTRALDISTR.centraldistr%TYPE,
                                      wactivat       IN  CENTRALPROD_CENTRALDISTR.activat%TYPE,
                                      wusuari        IN  CENTRALPROD_CENTRALDISTR.usuarialta%TYPE,
                                      RTN            OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaCentralProdCentralDistr
FUNCIO:          Alta de les connexions de linies de comunicaci i centrals de distribuci que surten d'una central de producci
PARMETRES  IN:  centralprod: Codi de la central de produccio
                 linia:  codi de la linia de comunicaci
                 centraldistr: Codi de la central de distribuci
                 activat: connexio activa ('S', 'N'), una central de distribuci pot tenir nomes una connexio
                          linia-central producci activa 
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CENTRALPRDLIN_JA_EXISTENT EXCEPTION;
  CODI_CONNCTRALDISTR_JA_ACTIVA EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaCentralProdCentralDistr';
    xdades_ent := 'Central Prod.: '    || wcentralprod  ||
                  '. Linia com.: '     || wlinia        ||
                  '. Central Distr.: ' || wcentraldistr ||
                  '. Activat: '        || wactivat      ||
                  '. Usuari: '         || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcentralprod IS NULL) OR (wlinia IS NULL) OR 
      (wcentraldistr IS NULL) OR (wactivat IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio relacio central/linia/distr NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CENTRALPROD_CENTRALDISTR
    WHERE centralprod = NLS_UPPER(TRIM(wcentralprod))
    AND linia = NLS_UPPER(TRIM(wlinia))
    AND centraldistr = NLS_UPPER(TRIM(wcentraldistr));
 
   IF (wcount > 0) THEN
      RAISE CODI_CENTRALPRDLIN_JA_EXISTENT;
   END IF;

-- Validacio una central distribuci te nomes una connexio central/linia activa

  IF (wactivat = 'S') THEN
     SELECT COUNT(*) 
      INTO wcount
       FROM CENTRALPROD_CENTRALDISTR
      WHERE centraldistr = NLS_UPPER(TRIM(wcentraldistr))
      AND activat = 'S' ;
   
     IF (wcount > 0) THEN
        RAISE CODI_CONNCTRALDISTR_JA_ACTIVA;
     END IF;
  END IF; 
-- Insercio de les dades
 
    INSERT INTO CENTRALPROD_CENTRALDISTR VALUES(NLS_UPPER(TRIM(wcentralprod)),
                            NLS_UPPER(TRIM(wlinia)),
                            NLS_UPPER(TRIM(wcentraldistr)),
                            NLS_UPPER(TRIM(wactivat)),
                            SYSDATE,
                            NULL,
                            NULL,
                            NLS_UPPER(TRIM(wusuari)),
                            NULL,
                            NULL);

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
 
         WHEN  CODI_CENTRALPRDLIN_JA_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Central Prod. ja connectat a la linia de comunicacio i a la central de distribuci';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
       
        WHEN  CODI_CONNCTRALDISTR_JA_ACTIVA  THEN
          xid_error := -003;
          xrtn  := 'KO';
          xdades_sort := 'Central Distr. te ja una connexi activa a una central de producci';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
     
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure ALTACLIENT
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTACLIENT" (wcodi       IN  CLIENT.codi%TYPE,
                     wtipus      IN  CLIENT.tipus%TYPE,
                     wnom        IN  CLIENT.nom%TYPE,
                     wcognom1    IN  CLIENT.cognom1%TYPE,
                     wcognom2    IN  CLIENT.cognom2%TYPE,
                     wadreca     IN  CLIENT.adreca%TYPE,
                     wpoblacio   IN  CLIENT.poblacio%TYPE,
                     wcodipostal IN  CLIENT.codipostal%TYPE,
                     wpais       IN  CLIENT.pais%TYPE,
                     wtelefon    IN  CLIENT.telefon%TYPE,
                     wemail      IN  CLIENT.email%TYPE,
                     wusuari     IN  CLIENT.usuarialta%TYPE,
                     RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaClient
FUNCIO:          Alta dels clients
PARMETRES  IN:  codi: Codi client (NIF o DNI)
                 nom: nom 
                 cognom1: primer cognom 
                 cognom2: segon cognom 
                 adreca:  adrea del client
                 poblacio: poblacio del client
                 codipostal: codi postal del client
                 pais: pais del client
                 telefon: telefon del client
                 email: email del client
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CLIENT_JA_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaClient';
    xdades_ent := 'Codi: '           || wcodi       ||
                  '. Tipus: '        || wtipus      ||
                  '. Nom: '          || wnom        ||
                  '. Cognom 1: '     || wcognom1    ||
                  '. Cognom 2: '     || wcognom2    ||
                  '. Adreca: '       || wadreca     ||
                  '. Poblacio: '     || wpoblacio   ||
                  '. Codi Postal: '  || wcodipostal ||
                  '. Pais: '         || wpais       ||
                  '. Telefon: '      || wtelefon    ||
                  '. Email: '        || wemail      ||
                  '. Usuari: '       || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wtipus IS NULL)  OR (wnom IS NULL) 
                      OR (wadreca IS NULL) OR (wpoblacio IS NULL)
                      OR (wcodipostal IS NULL) OR (wpais IS NULL) 
                      OR (wtelefon IS NULL) OR (wemail IS NULL) 
                      OR (wusuari IS NULL)THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;    
  
   IF (wtipus = 'F') AND ((wcognom1 IS NULL) OR (wcognom2 IS NULL)) THEN   
      RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
   
  
-- Validacio Client no existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CLIENT
    WHERE codi = NLS_UPPER(TRIM(wcodi));
 
    
   IF (wcount > 0) THEN
      RAISE CODI_CLIENT_JA_EXISTENT;
   END IF;

-- Insercio de les dades
 
    INSERT INTO CLIENT VALUES(NLS_UPPER(TRIM(wcodi)),
                              NLS_UPPER(TRIM(wtipus)),
                              NLS_UPPER(TRIM(wnom)),
                              NLS_UPPER(TRIM(wcognom1)),
                              NLS_UPPER(TRIM(wcognom2)),
                              NLS_UPPER(TRIM(wadreca)),
                              NLS_UPPER(TRIM(wpoblacio)),
                              wcodipostal,
                              NLS_UPPER(TRIM(wpais)),
                              NLS_UPPER(TRIM(wtelefon)),
                              NLS_UPPER(TRIM(wemail)),
                              SYSDATE,
                              NULL,
                              NULL,
                              NLS_UPPER(TRIM(wusuari)),
                              NULL,
                               NULL);

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_CLIENT_JA_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi client ja existent';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure ALTACOMPTADOR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTACOMPTADOR" (wcodi        IN  COMPTADOR.codi%TYPE,
                        widmodel     IN  COMPTADOR.idmodel%TYPE,
                        wadreca      IN  COMPTADOR.adreca%TYPE,
                        wpoblacio    IN  COMPTADOR.poblacio%TYPE,
                        wcodipostal  IN  COMPTADOR.codipostal%TYPE,
                        wpais        IN  COMPTADOR.pais%TYPE,
                        wconsum      IN  COMPTADOR.consum%TYPE,
                        wusuari      IN  COMPTADOR.usuarialta%TYPE,
                        RTN         OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaComptador
FUNCIO:          Alta dels comptadors
PARMETRES  IN:  codi: Codi del comptador
                 idmodel: model del comptador
                 adreca:  adrea on esta situat el comptador
                 poblacio: poblacio on esta situat el comptador
                 codipostal: codi postal on esta situat el comptador
                 pais: pais on esta situat el comptador
                 consum: consum del comptador
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
  usuari        char(10);
  
-- Dades de treball per al procediment
  wcount       NUMBER;

-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_COMPTADOR_JA_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaComptador';
    xdades_ent := 'Codi: '           || wcodi       ||
                  '. Model: '        || widmodel    ||  
                  '. Adreca: '       || wadreca     ||
                  '. Poblacio: '     || wpoblacio   ||
                  '. Codi Postal: '  || wcodipostal ||
                  '. Pais: '         || wpais       ||
                  '. Consum: '       || wconsum     ||
                  '. Usuari: '       || wusuari ;  
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi is null) OR (widmodel IS NULL) OR (wadreca IS NULL)
                      OR (wpoblacio IS NULL) OR (wcodipostal IS NULL) 
                      OR (wpais IS NULL)  OR (wconsum IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio camp PAIS no existent

   SELECT COUNT(*) 
    INTO wcount
     FROM COMPTADOR
    WHERE codi = NLS_UPPER(TRIM(wcodi));
 
    
   IF (wcount > 0) THEN
      RAISE CODI_COMPTADOR_JA_EXISTENT;
   END IF;

-- Insercio de les dades
 
    INSERT INTO COMPTADOR VALUES(NLS_UPPER(TRIM(wcodi)) , 
                                 NLS_UPPER(TRIM(widmodel)),  
                                 NLS_UPPER(TRIM(wadreca)),
                                 NLS_UPPER(TRIM(wpoblacio)),
                                 wcodipostal,
                                 NLS_UPPER(TRIM(wpais)),
                                 wconsum,
                                 0,
                                 SYSDATE,
                                 NULL,
                                 NULL,
                                 NLS_UPPER(TRIM(wusuari)),
                                 NULL,
                                 NULL);

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_COMPTADOR_JA_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi comptador ja existent';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure ALTACOMPTADORCENTRALDISTR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTACOMPTADORCENTRALDISTR" (wcomptador    IN  COMPTADOR_CENTRALDISTR.comptador%TYPE,
                                    wcentraldistr IN  COMPTADOR_CENTRALDISTR.centraldistr%TYPE,
                                    wactivat      IN  COMPTADOR_CENTRALDISTR.activat%TYPE,
                                    wusuari       IN  COMPTADOR_CENTRALDISTR.usuarialta%TYPE,
                                    RTN           OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaComptadorCentralDistr
FUNCIO:          Alta de les centrals de distribuci a les que esta conectat un comptador (1 a n)
PARMETRES  IN:  comptador: Codi de comptador
                 centraldistr:  codi de la central de distribuci
                 activat: connexio activa ('S', 'N'), un comptador pot estar connectat a mes d'una central distr. pero 
                          nomes una connexio pot estar activa
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CMPTDRCTDISTR_JA_EXISTENT EXCEPTION;
  CODI_CMPTDRCTDISTR_JA_ACTIU EXCEPTION;
  CODI_CMPTDRCTDISTR_NO_ACTIU EXCEPTION;
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaComptadorCentralDistr';
    xdades_ent := 'Comptador: '         || wcomptador    ||
                  '. Central Distr.: '  || wcentraldistr ||
                  '. Connexi activa: ' || wactivat ||
                  '. Usuari: '          || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcomptador IS NULL) OR (wcentraldistr IS NULL) OR 
      (wactivat IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio relacio comptador/central distribuci NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM COMPTADOR_CENTRALDISTR
    WHERE comptador = NLS_UPPER(TRIM(wcomptador))
    AND centraldistr = NLS_UPPER(TRIM(wcentraldistr));
 
    
   IF (wcount > 0) THEN
      RAISE CODI_CMPTDRCTDISTR_JA_EXISTENT;
   END IF;

-- Validacio comptador ja te una central distribuci activada, nomes pot haver una

IF (wactivat = 'S') THEN
   SELECT COUNT(*) 
    INTO wcount
     FROM COMPTADOR_CENTRALDISTR
    WHERE comptador = NLS_UPPER(TRIM(wcomptador))
    AND activat = 'S';
    
   IF (wcount > 0) THEN
      RAISE CODI_CMPTDRCTDISTR_JA_ACTIU;
   END IF;

END IF;
-- Insercio de les dades
 
    INSERT INTO COMPTADOR_CENTRALDISTR VALUES(NLS_UPPER(TRIM(wcomptador)),
                            NLS_UPPER(TRIM(wcentraldistr)),
                            NLS_UPPER(TRIM(wactivat)),
                            SYSDATE,
                            NULL,
                            NULL,
                            NLS_UPPER(TRIM(wusuari)),
                            NULL,
                            NULL);

-- Validacio comptador te una central distribuci activada, es un warning
    SELECT COUNT(*) 
      INTO wcount
       FROM COMPTADOR_CENTRALDISTR
      WHERE comptador = NLS_UPPER(TRIM(wcomptador))
      AND activat = 'S';
    
     IF (wcount = 0) THEN
        RAISE CODI_CMPTDRCTDISTR_NO_ACTIU;
     END IF;

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
   
         WHEN  CODI_CMPTDRCTDISTR_JA_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Comptador ja connectat a la Central de distribucio';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN  CODI_CMPTDRCTDISTR_JA_ACTIU  THEN
          xid_error := -003;
          xrtn  := 'KO';
          xdades_sort := 'Comptador ja connectat a una Central de distribucio activa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
   
         WHEN  CODI_CMPTDRCTDISTR_NO_ACTIU  THEN
          xid_error := -004;
          xrtn  := 'OK';
          xdades_sort := 'Atenci: Comptador no te cap Central de distribucio activa';
          RTN         := 'WARNING.' || xdades_sort;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
 
         WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure ALTACONTRACTE
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTACONTRACTE" (wcodi      IN  CONTRACTE.codi%TYPE,
                        wdataini   IN  CONTRACTE.datainici%TYPE,
                        wcomptador IN  CONTRACTE.comptador%TYPE,
                        wclient    IN  CONTRACTE.client%TYPE,
                        wpotencia  IN  CONTRACTE.potencia%TYPE,
                        wpreukwh   IN  CONTRACTE.preukwh%TYPE,
                        wusuari    IN  CONTRACTE.usuarialta%TYPE,
                        RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaContracte
FUNCIO:          alta dels contractes
PARMETRES  IN:  codi: Codi del contracte
                 datainici:  data d'inici del contracte
                 comptador:  codi del comptador associat al contracte
                 client:  codi del client associat al contracte
                 potencia:  potencia conctractada 
                 preukwh:  preu del kwH
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(25);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
    
-- Dades de treball per al procediment
  wcount       NUMBER;

-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CONTRACTE_JA_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaContracte';
    xdades_ent := 'Codi: '         || wcodi     ||
                  '. Data inici: ' || wdataini  ||  
                  '. Comptador: '  || wcomptador||  
                  '. Client: '     || wclient   ||  
                  '. Potencia: '   || wpotencia ||  
                  '. Preu Kwh: '   || wpreukwh  ||
                  '. Usuari: '     || wusuari ;   
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wdataini IS NULL) OR (wcomptador IS NULL) 
                      OR (wclient IS NULL) OR (wpotencia IS NULL) 
                      OR (wpreukwh IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio camp PAIS NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CONTRACTE
    WHERE codi = NLS_UPPER(TRIM(wcodi));
       
   IF (wcount > 0) THEN
      RAISE CODI_CONTRACTE_JA_EXISTENT;
   END IF;

-- Insercio de les dades
 
    INSERT INTO CONTRACTE VALUES(NLS_UPPER(TRIM(wcodi)) , 
                                wdataini,  
                                null,  
                                wcomptador,
                                wclient,
                                wpotencia,
                                wpreukwh,
                                SYSDATE,
                                NULL,
                                NULL,
                                NLS_UPPER(TRIM(wusuari)),
                                NULL,
                                NULL);

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
          
         WHEN  CODI_CONTRACTE_JA_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi contracte ja existent';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
                
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure ALTALECTURA
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTALECTURA" (wdata        IN  LECTURA.datalectura%TYPE,
                      wcomptador   IN  LECTURA.comptador%TYPE,
                      wvalor       IN  LECTURA.valor%TYPE,
                      wtipus       IN  LECTURA.tipus%TYPE,
                      wusuari      IN  LECTURA.usuarialta%TYPE,
                      RTN         OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaLectura
FUNCIO:          Alta de la lectura d'un comptador
PARMETRES  IN:  data: data de la lectura
                 comptador:  codi del comptador
                 valor: valor de la lectura
                 tipus: modalitat de lectura (presencial 'P' o telematica 'T')
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(25);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
  wconsum      NUMBER;
  wcentraldistr VARCHAR(12);
  wlectura     NUMBER;
  wlecturaold  NUMBER (7,2);
  wconsumold   NUMBER;
  wlinia VARCHAR(12);
  wcentralprod VARCHAR(12);
  
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_LECTURA_JA_EXISTENT EXCEPTION;
  CODI_CMPTDRCTDISTR_NO_ACTIU EXCEPTION;
  CODI_LINIA_NO_EXISTEIX EXCEPTION;
  CODI_CENTRALPROD_NO_EXISTEIX EXCEPTION;
  CODI_COMPTADOR_NO_EXISTEIX EXCEPTION;
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaLectura';
    xdades_ent := 'Data: '                || wdata        ||
                  '. Comptador: '         || wcomptador   ||
                  '. Lectura: '           || wvalor       ||
                  '. Modalitat (T, P): '  || wtipus       ||
                  '. Usuari: '            || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wdata IS NULL) OR (wcomptador IS NULL) OR (wvalor IS NULL)
                      OR (wtipus IS NULL) OR  (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 --OR (wcentralprod IS NULL) 
-- Validacio camp LECTURA no existent

   SELECT COUNT(*) 
    INTO wcount
     FROM LECTURA
    WHERE datalectura = wdata
     AND comptador = NLS_UPPER(TRIM(wcomptador));
 
    
   IF (wcount > 0) THEN
      RAISE CODI_LECTURA_JA_EXISTENT;
   END IF;
   
-- Cerca de la central distribuci activa pel comptador

     SELECT centraldistr 
      INTO wcentraldistr
       FROM COMPTADOR_CENTRALDISTR
      WHERE comptador = NLS_UPPER(TRIM(wcomptador))
      AND activat = 'S';
    
     IF (wcentraldistr IS NULL) THEN
        RAISE CODI_CMPTDRCTDISTR_NO_ACTIU;
     END IF;

-- Cerca de la linia i central de produccio a la que esta connectat la central de distribuci
 
    SELECT centralprod, linia
    INTO wcentralprod, wlinia
     FROM  CENTRALPROD_CENTRALDISTR 
    WHERE centraldistr = wcentraldistr
    AND activat = 'S';
    
   IF (wcentralprod IS NULL) THEN
        RAISE CODI_CENTRALPROD_NO_EXISTEIX;
   END IF;
 
 -- Calcul del consum, que es la diferencia entre el valor de la ultima lectura i l'anterior 
   
   SELECT lectura, consum
    INTO wlecturaold, wconsumold
     FROM COMPTADOR
    WHERE codi = NLS_UPPER(TRIM(wcomptador));
    
   IF (wcentralprod IS NULL) THEN
        RAISE CODI_COMPTADOR_NO_EXISTEIX;
   END IF;
    
    wconsum := wvalor - wlecturaold;
 
  -- Insercio de les dades
    INSERT INTO LECTURA VALUES(wdata,
                               NLS_UPPER(TRIM(wcomptador)),
                               NLS_UPPER(TRIM(wcentraldistr)),
                               NLS_UPPER(TRIM(wlinia)),
                               NLS_UPPER(TRIM(wcentralprod)),
                               wvalor,
                               wconsum,
                               NLS_UPPER(TRIM(wtipus)),
                               SYSDATE,
                               NULL,
                               NULL,
                               NLS_UPPER(TRIM(wusuari)),
                               NULL,
                               NULL);
                               
                               
     UPDATE COMPTADOR 
      SET lectura = wvalor,
          consum = consum + wconsum
      WHERE codi = NLS_UPPER(TRIM(wcomptador));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_LECTURA_JA_EXISTENT THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Lectura ja existent';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
         WHEN  CODI_CMPTDRCTDISTR_NO_ACTIU  THEN
          xid_error := -003;
          xrtn  := 'KO';
          xdades_sort := 'Comptador no te cap Central de distribucio activa, no es pot atribuir la lectura';
          RTN         := 'ERROR.' || xdades_sort;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
         WHEN  CODI_LINIA_NO_EXISTEIX  THEN
          xid_error := -004;
          xrtn  := 'KO';
          xdades_sort := 'Comptador no te cap linia associada a la Central de distribucio activa, no es pot atribuir la lectura';
          RTN         := 'ERROR.' || xdades_sort;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
         WHEN  CODI_CENTRALPROD_NO_EXISTEIX  THEN
          xid_error := -005;
          xrtn  := 'KO';
          xdades_sort := 'Comptador no te cap Central de Producci associada a la Linia-Central de distribucio activa, no es pot atribuir la lectura';
          RTN         := 'ERROR.' || xdades_sort;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN  CODI_COMPTADOR_NO_EXISTEIX  THEN
          xid_error := -005;
          xrtn  := 'KO';
          xdades_sort := 'No s''ha trobat el valor de la lectura anterior del Comptador';
          RTN         := 'ERROR.' || xdades_sort;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
           
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure ALTALINIACOM
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTALINIACOM" (wcodi       IN  LINIA_COM.codi%TYPE,
                       wcapacitat  IN  LINIA_COM.capacitatmax%TYPE,
                       wusuari     IN  LINIA_COM.usuarialta%TYPE,
                   RTN      OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaLiniaCom
FUNCIO:          Alta de les linies de comunicaci.
PARMETRES  IN:  codi: Codi de la linia
                 capacitatmax:  capacitat maxima de transport de la linia
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(25);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_LINIA_JA_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaLiniaCom';
    xdades_ent := 'Codi: '       || wcodi  ||
                  '.Capacitat: ' || wcapacitat   ||
                  '. Usuari: '   || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wcapacitat IS NULL) or (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio camp LINIA NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM LINIA_COM
    WHERE codi = NLS_UPPER(TRIM(wcodi));
 
    
   IF (wcount > 0) THEN
      RAISE CODI_LINIA_JA_EXISTENT;
   END IF;

-- Insercio de les dades
 
    INSERT INTO LINIA_COM VALUES(NLS_UPPER(TRIM(wcodi)),
                            wcapacitat,
                            SYSDATE,
                            NULL,
                            NULL,
                            NLS_UPPER(TRIM(wusuari)),
                            NULL,
                            NULL);

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_LINIA_JA_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi linia de comunicaci ja existent';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
 
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure ALTAMODEL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTAMODEL" (wcodi     IN  MODELCMPT.codi%TYPE,
                    wpotencia IN  MODELCMPT.potenciamax%TYPE,
                    wempresa  IN  MODELCMPT.empresa%TYPE,
                    wanyfab   IN  MODELCMPT.anyfab%TYPE,
                    wusuari   IN  MODELCMPT.usuarialta%TYPE,
                    RTN      OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaModel
FUNCIO:          alta dels models dels comptadors 
PARMETRES  IN:  codi: Codi del model
                 potenciamax:  potencia maxima 
                 empresa:  empresa de fabricacio
                 anyfab:   any de fabricaci
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(25);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
    
-- Dades de treball per al procediment
  wcount       NUMBER;

-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_MODEL_JA_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaModel';
    xdades_ent := 'Codi: '        || wcodi     ||
                  '. Potencia: '  || wpotencia ||  
                  '. Empresa: '   || wempresa  ||
                  '. Any fab.: '  || wanyfab   ||
                  '. Usuari: '    || wusuari ;   
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wpotencia IS NULL) OR (wempresa IS NULL) OR (wanyfab IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio camp PAIS NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM MODELCMPT
    WHERE codi = NLS_UPPER(TRIM(wcodi));
 
    
   IF (wcount > 0) THEN
      RAISE CODI_MODEL_JA_EXISTENT;
   END IF;

-- Insercio de les dades
 
    INSERT INTO MODELCMPT VALUES(NLS_UPPER(TRIM(wcodi)) , 
                                wpotencia,  
                                NLS_UPPER(TRIM(wempresa)),
                                wanyfab,
                                SYSDATE,
                                NULL,
                                NULL,
                                NLS_UPPER(TRIM(wusuari)),
                                NULL,
                                NULL);

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
   
         WHEN  CODI_MODEL_JA_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi model ja existent';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
       
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure ALTAPAIS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTAPAIS" (wcodi     IN  PAIS.codi%TYPE,
                   wnom      IN  PAIS.nom%TYPE,
                   wusuari   IN  PAIS.usuarialta%TYPE,
                   RTN      OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             altaPais
FUNCIO:          Alta dels paisos.
PARMETRES  IN:  codi: Codi ISO del pais
                 nom:  nom del pais
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(25);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_PAIS_JA_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'altaPais';
    xdades_ent := 'Codi: '      || wcodi  ||
                  '. Nom: '     || wnom   ||
                  '. Usuari: '  || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wnom IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio camp PAIS NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM PAIS
    WHERE codi = NLS_UPPER(TRIM(wcodi));
 
    
   IF (wcount > 0) THEN
      RAISE CODI_PAIS_JA_EXISTENT;
   END IF;

-- Insercio de les dades
 
    INSERT INTO PAIS VALUES(NLS_UPPER(TRIM(wcodi)),
                            NLS_UPPER(TRIM(wnom)),
                            SYSDATE,
                            NULL,
                            NULL,
                            NLS_UPPER(TRIM(wusuari)),
                            NULL,
                            NULL);

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
         WHEN  CODI_PAIS_JA_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi pais ja existent';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
 
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure BAIXACENTRALDISTR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXACENTRALDISTR" (wcodi       IN  CENTRAL_DISTR.codi%TYPE,
                            wusuari     IN  CENTRAL_DISTR.usuarialta%TYPE,
                            RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             baixaCentralDistr
FUNCIO:          Baixa logica de les centrals de distribucio, no esborra fisicament
PARMETRES  IN:  codi: Codi de la central de distribucio
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(25);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CENTRALDISTR_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'baixaCentralDistr';
    xdades_ent := 'Codi: '           || wcodi       ||
                  '. Usuari: '       || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR  (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio camp CENTRAL DISTRIBUCIO no existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CENTRAL_DISTR
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CENTRALDISTR_NO_EXISTENT;
   END IF;

-- baixa logica de les dades
 
    UPDATE CENTRAL_DISTR 
    SET databaixa   = SYSDATE,
        usuaribaixa = NLS_UPPER(TRIM(wusuari))
    WHERE CODI = NLS_UPPER(TRIM(wcodi));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_CENTRALDISTR_NO_EXISTENT THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi central distribuci no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure BAIXACENTRALPROD
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXACENTRALPROD" (wcodi       IN  CENTRAL_PROD.codi%TYPE,
                                       wusuari     IN  CENTRAL_PROD.usuarialta%TYPE,
                                        RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             baixaCentralProd
FUNCIO:          Baixa logica de les centrals de produccio, no s'esborra fisicament
PARMETRES  IN:  codi: Codi de la central
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(25);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CENTRALPROD_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'baixaCentralProd';
    xdades_ent := 'Codi: '          || wcodi       ||
                  '. Usuari: '      || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR  (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio camp CENTRAL PRODUCCIO NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CENTRAL_PROD
    WHERE codi = NLS_UPPER(TRIM(wcodi))
     AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CENTRALPROD_NO_EXISTENT;
   END IF;

-- baixa logica de les dades
 
    UPDATE CENTRAL_PROD 
    SET databaixa    = SYSDATE,
        usuaribaixa  = NLS_UPPER(TRIM(wusuari))
    WHERE CODI = NLS_UPPER(TRIM(wcodi));
    
    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
          
         WHEN  CODI_CENTRALPROD_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi central produci no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure BAIXACENTRALPRODCENTRALDISTR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXACENTRALPRODCENTRALDISTR" (wcentralprod   IN  CENTRALPROD_CENTRALDISTR.centralprod%TYPE,
                                       wlinia         IN  CENTRALPROD_CENTRALDISTR.linia%TYPE,
                                       wcentraLdistr  IN  CENTRALPROD_CENTRALDISTR.centraldistr%TYPE,
                                       wusuari        IN  CENTRALPROD_CENTRALDISTR.usuarialta%TYPE,
                                       RTN      OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             baixaCentralProdCentralDistr
FUNCIO:          Baixa logica d'una connexio linia de comunicaci/central distribucio d'una central de producci 
PARMETRES  IN:  centralprod: Codi de la central de produccio
                 linia:  codi de la linia de comunicaci
                 centraldistr: Codi de la central de distribuci
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(35);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CENTRALPRDLIN_NO_EXISTENT EXCEPTION;
  CODI_CONNCTRALDISTR_NO_ACTIVA EXCEPTION;
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'baixaCentralProdCentralDistr';
    xdades_ent := 'Central Prod.: '    || wcentralprod  ||
                  '. Linia com.: '     || wlinia        ||
                  '. Central Distr.: ' || wcentraldistr ||
                  '. Usuari: '         || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcentralprod IS NULL) OR (wlinia IS NULL) OR 
      (wcentraldistr IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio relacio/linia existeix

   SELECT COUNT(*) 
    INTO wcount
     FROM CENTRALPROD_CENTRALDISTR
    WHERE centralprod = NLS_UPPER(TRIM(wcentralprod))
    AND linia = NLS_UPPER(TRIM(wlinia))
    AND centraldistr = NLS_UPPER(TRIM(wcentraldistr))
    AND databaixa is null;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CENTRALPRDLIN_NO_EXISTENT;
   END IF;

-- baixa logica de les dades
 
    UPDATE CENTRALPROD_CENTRALDISTR 
    SET databaixa   = SYSDATE,
        usuaribaixa = NLS_UPPER(TRIM(wusuari))
    WHERE centralprod = NLS_UPPER(TRIM(wcentralprod))
    AND linia = NLS_UPPER(TRIM(wlinia))
    AND centraldistr = NLS_UPPER(TRIM(wcentraldistr));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

-- Validacio central distribuci te al menys una connexi activada, es un warning
    SELECT COUNT(*) 
      INTO wcount
       FROM CENTRALPROD_CENTRALDISTR
      WHERE centraldistr = NLS_UPPER(TRIM(wcentraldistr))
      AND activat = 'S';
    
     IF (wcount = 0) THEN
        RAISE CODI_CONNCTRALDISTR_NO_ACTIVA;
     END IF;
 
    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_CENTRALPRDLIN_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Central Prod. no associat a la linia de comunicacio/central distribuci o ja donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
       WHEN  CODI_CONNCTRALDISTR_NO_ACTIVA  THEN
          xid_error := -003;
          xrtn  := 'OK';
          xdades_sort := 'Warning: Central Distr. no te una connexi activa a una central de producci';
          RTN         := 'WARNING.' || xdades_sort;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
       
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure BAIXACLIENT
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXACLIENT" (wcodi       IN  CLIENT.codi%TYPE,
                      wusuari     IN  CLIENT.usuarialta%TYPE,
                      RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             baixaClient
FUNCIO:          baixa logica dels clients, no s'esborra fisicament
PARMETRES  IN:  codi: Codi client (NIF o DNI)
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(25);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CLIENT_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'baixaClient';
    xdades_ent := 'Codi: '           || wcodi       ||
                  '. Usuari: '       || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wusuari IS NULL)THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio Client no existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CLIENT
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CLIENT_NO_EXISTENT;
   END IF;

-- baixa logica de les dades
 
    UPDATE CLIENT 
    SET databaixa   = SYSDATE,
        usuaribaixa = NLS_UPPER(TRIM(wusuari))
    WHERE codi = NLS_UPPER(TRIM(wcodi));
   
    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_CLIENT_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi client no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure BAIXACOMPTADOR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXACOMPTADOR" (wcodi        IN  COMPTADOR.codi%TYPE,
                         wusuari      IN  COMPTADOR.usuaribaixa%TYPE,
                         RTN         OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             baixaComptador
FUNCIO:          baixa logica dels comptadors, no s'esborra fisicament
PARMETRES  IN:  codi: Codi del comptador
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(25);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
  usuari        char(10);
  
-- Dades de treball per al procediment
  wcount       NUMBER;

-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_COMPTADOR_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'baixaComptador';
    xdades_ent := 'Codi: '       || wcodi  ||
                  '. Usuari: '   || wusuari ;  
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi is null) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio COMPTADOR NO existent

   SELECT COUNT(*) 
    INTO wcount
    FROM COMPTADOR
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_COMPTADOR_NO_EXISTENT;
   END IF;

-- baixa logica de les dades
 
    UPDATE COMPTADOR 
    SET databaixa    = SYSDATE,
        usuaribaixa  = NLS_UPPER(TRIM(wusuari))
    WHERE CODI = NLS_UPPER(TRIM(wcodi));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, usuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_COMPTADOR_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi comptador no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, usuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, usuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure BAIXACOMPTADORCENTRALDISTR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXACOMPTADORCENTRALDISTR" (wcomptador    IN  COMPTADOR_CENTRALDISTR.comptador%TYPE,
                                     wcentraldistr IN  COMPTADOR_CENTRALDISTR.centraldistr%TYPE,
                                     wusuari       IN  COMPTADOR_CENTRALDISTR.usuarialta%TYPE,
                                     RTN           OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             baixaComptadorCentralDistr
FUNCIO:          baixa logica de la connexio del comptador i la centrals de distribuci
PARMETRES  IN:  comptador: Codi de comptador
                 centraldistr:  codi de la central de distribuci
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CMPTDRCTDISTR_NO_EXISTENT EXCEPTION;
  CODI_CMPTDRCTDISTR_NO_ACTIU EXCEPTION;
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'baixaComptadorCentralDistr';
    xdades_ent := 'Comptador: '        || wcomptador    ||
                  '. Central Distr.: ' || wcentraldistr ||
                  '. Usuari: '         || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcomptador IS NULL) OR (wcentraldistr IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio relacio comptador/central distribuci NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM COMPTADOR_CENTRALDISTR
    WHERE comptador = NLS_UPPER(TRIM(wcomptador))
    AND centraldistr = NLS_UPPER(TRIM(wcentraldistr))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CMPTDRCTDISTR_NO_EXISTENT;
   END IF;

-- Baixa logica de la relacio comptador/central distribuci
 
   UPDATE COMPTADOR_CENTRALDISTR 
     SET databaixa    = SYSDATE,
         usuaribaixa  = NLS_UPPER(TRIM(wusuari))
    WHERE comptador   = NLS_UPPER(TRIM(wcomptador));
    
    
-- Validacio comptador te una central distribuci activada, es un warning
    SELECT COUNT(*) 
      INTO wcount
       FROM COMPTADOR_CENTRALDISTR
      WHERE comptador = NLS_UPPER(TRIM(wcomptador))
      AND activat = 'S';
    
     IF (wcount = 0) THEN
        RAISE CODI_CMPTDRCTDISTR_NO_ACTIU;
     END IF;
 
    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_CMPTDRCTDISTR_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Comptador no connectat a la Central de distribucio o connexi donada de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN  CODI_CMPTDRCTDISTR_NO_ACTIU  THEN
          xid_error := -003;
          xrtn  := 'OK';
          xdades_sort := 'Warning: Comptador no te cap Central de distribucio activa';
          RTN         := 'WARNING.' || xdades_sort;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure BAIXACONTRACTE
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXACONTRACTE" (wcodi      IN  CONTRACTE.codi%TYPE,
                         wdatafi    IN  CONTRACTE.databaixa%TYPE,
                         wusuari    IN  CONTRACTE.usuarialta%TYPE,
                         RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             baixaContracte
FUNCIO:          baixa del contracte
PARMETRES  IN:  codi: Codi del contracte
                 datafi:  data de fi del contracte
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
    
-- Dades de treball per al procediment
  wcount       NUMBER;

-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CONTRACTE_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'baixaContracte';
    xdades_ent := 'Codi: '         || wcodi     ||
                  '. Data fi: '    || wdatafi   ||  
                  '. Usuari: '     || wusuari ;   
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wdatafi IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio codi contracte NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CONTRACTE
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CONTRACTE_NO_EXISTENT;
   END IF;

-- baixa logica de les dades
 
    UPDATE CONTRACTE 
    SET datafi      = wdatafi,  
        databaixa   = SYSDATE,
        usuaribaixa = NLS_UPPER(TRIM(wusuari))     
    WHERE codi = NLS_UPPER(TRIM(wcodi));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
          
        WHEN  CODI_CONTRACTE_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi contracte no existent o donat de baixa logica';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure BAIXALINIACOM
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXALINIACOM" (wcodi       IN  LINIA_COM.codi%TYPE,
                        wusuari     IN  LINIA_COM.usuarialta%TYPE,
                        RTN      OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             baixaLiniaCom
FUNCIO:          baixa logica de les linies de comunicaci, no s'esborra fisicament
PARMETRES  IN:  codi: Codi de la linia
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_LINIA_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'baixaLiniaCom';
    xdades_ent := 'Codi: '       || wcodi  ||
                  '. Usuari: '   || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio camp LINIA NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM LINIA_COM
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_LINIA_NO_EXISTENT;
   END IF;

-- baixa logica de les dades
 
    UPDATE LINIA_COM 
      SET databaixa    = SYSDATE,
          usuaribaixa  = NLS_UPPER(TRIM(wusuari))
      WHERE codi = NLS_UPPER(TRIM(wcodi));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_LINIA_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi linia de comunicaci no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure BAIXAMODEL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXAMODEL" (wcodi     IN  MODELCMPT.codi%TYPE,
                     wusuari   IN  MODELCMPT.usuaribaixa%TYPE,
                     RTN      OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             baixaModel
FUNCIO:          baixa logica dels models dels comptadors, no s'esborra fisicament
PARMETRES  IN:  codi: Codi del model
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
    
-- Dades de treball per al procediment
  wcount       NUMBER;

-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_MODEL_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'baixaModel';
    xdades_ent := 'Codi: '      || wcodi  ||
                  '. Usuari: '  || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio MODEL NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM MODELCMPT
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
     
   IF (wcount = 0) THEN
      RAISE CODI_MODEL_NO_EXISTENT;
   END IF;

-- baixa logica de les dades
 
    UPDATE MODELCMPT 
    SET databaixa   = SYSDATE,
        usuaribaixa = NLS_UPPER(TRIM(wusuari))
    WHERE codi      = NLS_UPPER(TRIM(wcodi));
 
    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_MODEL_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi model no existent o ja donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure BAIXAPAIS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXAPAIS" (wcodi     IN  PAIS.codi%TYPE,
                    wusuari   IN  PAIS.usuaribaixa%TYPE,
                    RTN      OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             baixaPais
FUNCIO:          baixa logica dels paisos, no s'esborren fisicament.
PARMETRES  IN:  codi: Codi ISO del pais
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_PAIS_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'baixaPais';
    xdades_ent := 'Codi: '     || wcodi ||
                  '. Usuari: ' || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) or (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio PAIS NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM PAIS
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_PAIS_NO_EXISTENT;
   END IF;


-- baixa logica de les dades

    UPDATE PAIS
       SET databaixa   = SYSDATE,
           usuaribaixa = NLS_UPPER(TRIM(wusuari))
     WHERE codi        = NLS_UPPER(TRIM(wcodi));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
       WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
         WHEN CODI_PAIS_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi pais no existent o ja donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/

--------------------------------------------------------
--  DDL for Procedure MODIFICACENTRALDISTR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MODIFICACENTRALDISTR" (wcodi       IN  CENTRAL_DISTR.codi%TYPE,
                               wadreca     IN  CENTRAL_DISTR.adreca%TYPE,
                               wpoblacio   IN  CENTRAL_DISTR.poblacio%TYPE,
                               wcodipostal IN  CENTRAL_DISTR.codipostal%TYPE,
                               wpais       IN  CENTRAL_DISTR.pais%TYPE,
                               wenergia    IN  CENTRAL_DISTR.energiamax%TYPE,
                               wusuari     IN  CENTRAL_DISTR.usuarialta%TYPE,
                               RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             modificaCentralDistr
FUNCIO:          Modificaci de les centrals de distribucio (totes les dades excepte el codi)
PARMETRES  IN:  codi: Codi de la central
                 adreca:  adrea on esta situada la central
                 poblacio: poblacio on esta situada la central
                 codipostal: codi postal on esta situada la central
                 pais: pais on esta situat situada la central
                 energiamax: maxim de energia electrica (KWh) que pot suministrar la central
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CENTRALDISTR_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'modificaCentralDistr';
    xdades_ent := 'Codi: '           || wcodi       ||
                  '. Adreca: '       || wadreca     ||
                  '. Poblacio: '     || wpoblacio   ||
                  '. Codi Postal: '  || wcodipostal ||
                  '. Pais: '         || wpais       ||
                  '. Energia '       || wenergia    ||
                  '. Usuari: '       || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wadreca IS NULL) OR (wpoblacio IS NULL)
                      OR (wcodipostal IS NULL) OR (wpais IS NULL) 
                      OR (wenergia IS NULL)  OR  (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio codi CENTRAL DISTRIBUCIO no existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CENTRAL_DISTR
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CENTRALDISTR_NO_EXISTENT;
   END IF;

-- Actualitzaci de les dades
 
    UPDATE CENTRAL_DISTR 
    SET adreca     = NLS_UPPER(TRIM(wadreca)),
        poblacio   = NLS_UPPER(TRIM(wpoblacio)),
        codipostal = wcodipostal,
        pais       = NLS_UPPER(TRIM(wpais)),
        energiamax = wenergia,
        datamod    = SYSDATE,
        usuarimod  = NLS_UPPER(TRIM(wusuari))
    WHERE CODI = NLS_UPPER(TRIM(wcodi));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_CENTRALDISTR_NO_EXISTENT THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi central distribuci no existent o donat de baix';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure MODIFICACENTRALPROD
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MODIFICACENTRALPROD" (wcodi       IN  CENTRAL_PROD.codi%TYPE,
                              wadreca     IN  CENTRAL_PROD.adreca%TYPE,
                              wpoblacio   IN  CENTRAL_PROD.poblacio%TYPE,
                              wcodipostal IN  CENTRAL_PROD.codipostal%TYPE,
                              wpais       IN  CENTRAL_PROD.pais%TYPE,
                              wtipus      IN  CENTRAL_PROD.tipus%TYPE,
                              wnummolins  IN  CENTRAL_PROD.nummolins%TYPE,
                              wnumpanels  IN  CENTRAL_PROD.numpanels%TYPE,
                              wkgCO2      IN  CENTRAL_PROD.kgCO2%TYPE,
                              wkgrebuig   IN  CENTRAL_PROD.kgrebuig%TYPE,
                              wenergiamin IN  CENTRAL_PROD.energiamin%TYPE,
                              wusuari     IN  CENTRAL_PROD.usuarialta%TYPE,
                              RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             modificaCentralProd
FUNCIO:          Modificacio de les centrals de produccio (tots els valor excepte el codi)
PARMETRES  IN:  codi: Codi de la central
                 adreca:  adrea on esta situada la central
                 poblacio: poblacio on esta situada la central
                 codipostal: codi postal on esta situada la central
                 pais: pais on esta situat situada la central
                 tipus: tipus de central (nuclear 'N', termica 'T', carbo 'C', eolica 'E', solar 'S'
                 nummolins: nombre molins de vent instalats per central eolica
                 numpanels: nombre panels instalats per central solar
                 kgCO2: Kilograms d'emisions de CO2 per centra termica i carbo
                 kgrebuig: Kilograms de rebuig radioactiu generat per central nuclear
                 energiamin: energia minima necesaria per funcionar per energia nuclear
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CENTRALPROD_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'modificaCentralProd';
    xdades_ent := 'Codi: '           || wcodi       ||
                  '. Adreca: '       || wadreca     ||
                  '. Poblacio: '     || wpoblacio   ||
                  '. Codi Postal: '  || wcodipostal ||
                  '. Pais: '         || wpais       ||
                  '. Tipus: '        || wtipus      ||
                  '. No. molins: '   || wnummolins  ||
                  '. No. panels: '   || wnumpanels  ||
                  '. Kg. CO2: '      || wkgCO2      ||
                  '. Kg. rebuig: '   || wkgrebuig   ||
                  '. Energia min. '  || wenergiamin ||
                  '. Usuari: '       || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wadreca IS NULL) OR (wpoblacio IS NULL)
                      OR (wcodipostal IS NULL) OR (wpais IS NULL) 
                      OR (wtipus IS NULL)  OR  (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
   
   IF (wtipus = 'E') AND (wnummolins IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
   IF (wtipus = 'S') AND (wnumpanels IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
   IF ((wtipus = 'T') OR (wtipus = 'C')) AND (wkgCO2 IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
   IF (wtipus = 'N') AND ((wkgrebuig IS NULL) OR (wenergiamin IS NULL)) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
 
-- Validacio codi CENTRAL PRODUCCIO NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CENTRAL_PROD
    WHERE codi = NLS_UPPER(TRIM(wcodi))
     AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CENTRALPROD_NO_EXISTENT;
   END IF;

-- Actualitzacio de les dades
 
    UPDATE CENTRAL_PROD 
    SET adreca     = NLS_UPPER(TRIM(wadreca)),
        poblacio   = NLS_UPPER(TRIM(wpoblacio)),
        codipostal = wcodipostal,
        pais       = NLS_UPPER(TRIM(wpais)),
        tipus      = wtipus,
        nummolins  = wnummolins,
        numpanels  = wnumpanels,
        kgCO2      = wkgCO2,
        kgrebuig   = wkgrebuig,
        energiamin = wenergiamin,
        datamod    = SYSDATE,
        usuarimod  = NLS_UPPER(TRIM(wusuari))
    WHERE CODI = NLS_UPPER(TRIM(wcodi));
    
    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_CENTRALPROD_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi central produci no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure MODIFICACLIENT
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MODIFICACLIENT" (wcodi       IN  CLIENT.codi%TYPE,
                         wtipus      IN  CLIENT.tipus%TYPE,
                         wnom        IN  CLIENT.nom%TYPE,
                         wcognom1    IN  CLIENT.cognom1%TYPE,
                         wcognom2    IN  CLIENT.cognom2%TYPE,
                         wadreca     IN  CLIENT.adreca%TYPE,
                         wpoblacio   IN  CLIENT.poblacio%TYPE,
                         wcodipostal IN  CLIENT.codipostal%TYPE,
                         wpais       IN  CLIENT.pais%TYPE,
                         wtelefon    IN  CLIENT.telefon%TYPE,
                         wemail      IN  CLIENT.email%TYPE,
                         wusuari     IN  CLIENT.usuarialta%TYPE,
                         RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             modificaClient
FUNCIO:          Modificaci dels clients (tot excepte el codi)
PARMETRES  IN:  codi: Codi client (NIF o DNI)
                 nom: nom 
                 cognom1: primer cognom 
                 cognom2: segon cognom 
                 adreca:  adrea del client
                 poblacio: poblacio del client
                 codipostal: codi postal del client
                 pais: pais del client
                 telefon: telefon del client
                 email: email del client
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CLIENT_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'modificaClient';
    xdades_ent := 'Codi: '           || wcodi       ||
                  '. Tipus: '        || wtipus      ||
                  '. Nom: '          || wnom        ||
                  '. Cognom 1: '     || wcognom1    ||
                  '. Cognom 2: '     || wcognom2    ||
                  '. Adreca: '       || wadreca     ||
                  '. Poblacio: '     || wpoblacio   ||
                  '. Codi Postal: '  || wcodipostal ||
                  '. Pais: '         || wpais       ||
                  '. Telefon: '      || wtelefon    ||
                  '. Email: '        || wemail      ||
                  '. Usuari: '       || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

    IF (wcodi IS NULL) OR (wtipus IS NULL)  OR (wnom IS NULL) 
                       OR (wadreca IS NULL) OR (wpoblacio IS NULL)
                       OR (wcodipostal IS NULL) OR (wpais IS NULL) 
                       OR (wtelefon IS NULL) OR (wemail IS NULL) 
                       OR (wusuari IS NULL)THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;    
  
   IF (wtipus = 'F') AND ((wcognom1 IS NULL) OR (wcognom2 IS NULL)) THEN   
      RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio codi Client no existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CLIENT
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CLIENT_NO_EXISTENT;
   END IF;

-- Actualitzaci de les dades
 
    UPDATE CLIENT 
    SET tipus      = NLS_UPPER(TRIM(wtipus)),
        nom        = NLS_UPPER(TRIM(wnom)),
        cognom1    = NLS_UPPER(TRIM(wcognom1)),
        cognom2    = NLS_UPPER(TRIM(wcognom2)),
        adreca     = NLS_UPPER(TRIM(wadreca)),
        poblacio   = NLS_UPPER(TRIM(wpoblacio)),
        codipostal = wcodipostal,
        pais       = NLS_UPPER(TRIM(wpais)),
        telefon    = wtelefon,
        email      = wemail,
        datamod    = SYSDATE,
        usuarimod  = NLS_UPPER(TRIM(wusuari))
    WHERE codi = NLS_UPPER(TRIM(wcodi));
   
    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_CLIENT_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi client no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure MODIFICACOMPTADOR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MODIFICACOMPTADOR" (wcodi        IN  COMPTADOR.codi%TYPE,
                            widmodel     IN  COMPTADOR.idmodel%TYPE,
                            wadreca      IN  COMPTADOR.adreca%TYPE,
                            wpoblacio    IN  COMPTADOR.poblacio%TYPE,
                            wcodipostal  IN  COMPTADOR.codipostal%TYPE,
                            wpais        IN  COMPTADOR.pais%TYPE,
                            wconsum      IN  COMPTADOR.consum%TYPE,
                            wusuari      IN  COMPTADOR.usuarimod%TYPE,
                            RTN         OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             modifIcaComptador
FUNCIO:          modifica dels comptadors (excepte el codi)
PARMETRES  IN:  codi: Codi del comptador
                 idmodel: model del comptador
                 adreca:  adrea on esta situat el comptador
                 poblacio: poblacio on esta situat el comptador
                 codipostal: codi postal on esta situat el comptador
                 pais: pais on esta situat el comptador
                 consum: consum del comptador
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
  usuari        char(10);
  
-- Dades de treball per al procediment
  wcount       NUMBER;

-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_COMPTADOR_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'modificaComptador';
    xdades_ent := 'Codi: '           || wcodi       ||
                  '. Model: '        || widmodel    ||  
                  '. Adreca: '       || wadreca     ||
                  '. Poblacio: '     || wpoblacio   ||
                  '. Codi Postal: '  || wcodipostal ||
                  '. Pais: '         || wpais       ||
                  '. Consum: '       || wconsum     ||
                  '. Usuari: '       || wusuari ;  
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi is null) OR (widmodel IS NULL) OR (wadreca IS NULL)
                      OR (wpoblacio IS NULL) OR (wcodipostal IS NULL) 
                      OR (wpais IS NULL)  OR (wconsum IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio codi COMPTADOR NO existent

   SELECT COUNT(*) 
    INTO wcount
    FROM COMPTADOR
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_COMPTADOR_NO_EXISTENT;
   END IF;

-- Actualitzacio de les dades
 
    UPDATE COMPTADOR 
    SET idmodel    = NLS_UPPER(TRIM(widmodel)),  
        adreca     = NLS_UPPER(TRIM(wadreca)),
        poblacio   = NLS_UPPER(TRIM(wpoblacio)),
        codipostal = wcodipostal,
        pais       = NLS_UPPER(TRIM(wpais)),
        consum     = wconsum,
        datamod    = SYSDATE,
        usuarimod  = NLS_UPPER(TRIM(wusuari))
    WHERE CODI = NLS_UPPER(TRIM(wcodi));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, usuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
          
        WHEN  CODI_COMPTADOR_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi comptador no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, usuari);
            
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, usuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure MODIFICACOMPTADORCENTRALDISTR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MODIFICACOMPTADORCENTRALDISTR" (wcomptador    IN  COMPTADOR_CENTRALDISTR.comptador%TYPE,
                                        wcentraldistr IN  COMPTADOR_CENTRALDISTR.centraldistr%TYPE,
                                        wactivat      IN  COMPTADOR_CENTRALDISTR.activat%TYPE,
                                        wusuari       IN  COMPTADOR_CENTRALDISTR.usuarialta%TYPE,
                                        RTN           OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             modificaComptadorCentralDistr
FUNCIO:          activa o desactiva la centrals de distribuci a la que esta conectat un comptador (nomes 1 activa)
PARMETRES  IN:  comptador: Codi de comptador
                 centraldistr:  codi de la central de distribuci
                 activat: connexio activa ('S', 'N')
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CMPTDRCTDISTR_NO_EXISTENT EXCEPTION;
  CODI_CMPTDRCTDISTR_JA_ACTIU EXCEPTION;
  CODI_CMPTDRCTDISTR_NO_ACTIU EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'modificaComptadorCentralDistr';
    xdades_ent := 'Comptador: '         || wcomptador    ||
                  '. Central Distr.: '  || wcentraldistr ||
                  '. Connexi activa: ' || wactivat ||
                  '. Usuari: '          || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcomptador IS NULL) OR (wcentraldistr IS NULL) OR 
      (wactivat IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio relacio comptador/central distribuci NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM COMPTADOR_CENTRALDISTR
    WHERE comptador = NLS_UPPER(TRIM(wcomptador))
    AND centraldistr = NLS_UPPER(TRIM(wcentraldistr))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CMPTDRCTDISTR_NO_EXISTENT;
   END IF;

-- Validacio comptador ja te una central distribuci activada, nomes pot haver una
  IF (wactivat = 'S') THEN
     SELECT COUNT(*) 
      INTO wcount
       FROM COMPTADOR_CENTRALDISTR
      WHERE comptador = NLS_UPPER(TRIM(wcomptador))
      AND activat = 'S';
    
     IF (wcount > 0) THEN
        RAISE CODI_CMPTDRCTDISTR_JA_ACTIU;
     END IF;
 END IF;
 
 
-- Actualitzaci de les dades
 
   UPDATE COMPTADOR_CENTRALDISTR 
     SET activat = NLS_UPPER(TRIM(wactivat)),
         datamod      = SYSDATE,
         usuarimod    = NLS_UPPER(TRIM(wusuari))
    WHERE comptador  = NLS_UPPER(TRIM(wcomptador))
    AND centraldistr = NLS_UPPER(TRIM(wcentraldistr));
    
-- Validacio comptador te una central distribuci activada, es un warning
    SELECT COUNT(*) 
      INTO wcount
       FROM COMPTADOR_CENTRALDISTR
      WHERE comptador = NLS_UPPER(TRIM(wcomptador))
      AND activat = 'S';
    
     IF (wcount = 0) THEN
        RAISE CODI_CMPTDRCTDISTR_NO_ACTIU;
     END IF;
 
    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_CMPTDRCTDISTR_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Comptador no connectat a la Central de distribucio o connexi donada de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN  CODI_CMPTDRCTDISTR_JA_ACTIU  THEN
          xid_error := -003;
          xrtn  := 'KO';
          xdades_sort := 'Comptador ja connectat a una Central de distribucio activa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
 
        WHEN  CODI_CMPTDRCTDISTR_NO_ACTIU  THEN
          xid_error := -004;
          xrtn  := 'OK';
          xdades_sort := 'Warning: Comptador no te cap Central de distribucio activa';
          RTN         := 'WARNING.' || xdades_sort;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
         
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure MODIFICACONTRACTE
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MODIFICACONTRACTE" (wcodi      IN  CONTRACTE.codi%TYPE,
                            wdataini   IN  CONTRACTE.datainici%TYPE,
                            wcomptador IN  CONTRACTE.comptador%TYPE,
                            wclient    IN  CONTRACTE.client%TYPE,
                            wpotencia  IN  CONTRACTE.potencia%TYPE,
                            wpreukwh   IN  CONTRACTE.preukwh%TYPE,
                            wusuari    IN  CONTRACTE.usuarialta%TYPE,
                            RTN        OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             modificaContracte
FUNCIO:          modifica de les dades del contracte
PARMETRES  IN:  codi: Codi del contracte
                 datainici:  data d'inici del contracte
                 comptador:  codi del comptador associat al contracte
                 client:  codi del client associat al contracte
                 potencia:  potencia conctractada 
                 preukwh:  preu del kwH
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
    
-- Dades de treball per al procediment
  wcount       NUMBER;

-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CONTRACTE_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'modificaContracte';
    xdades_ent := 'Codi: '         || wcodi     ||
                  '. Data inici: ' || wdataini  ||  
                  '. Comptador: '  || wcomptador||  
                  '. Client: '     || wclient   ||  
                  '. Potencia: '   || wpotencia ||  
                  '. Preu Kwh: '   || wpreukwh  ||
                  '. Usuari: '     || wusuari ;   
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wdataini IS NULL) OR (wcomptador IS NULL) 
                      OR (wclient IS NULL) OR (wpotencia IS NULL) 
                      OR (wpreukwh IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio codi Contracte NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CONTRACTE
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_CONTRACTE_NO_EXISTENT;
   END IF;

-- Actualitzaci de les dades
 
    UPDATE CONTRACTE 
    SET datainici = wdataini,  
        comptador = wcomptador,
        client    = wclient,
        potencia  = wpotencia,
        preukwh   = wpreukwh,
        datamod   = SYSDATE,
        usuarimod = NLS_UPPER(TRIM(wusuari))     
    WHERE codi = NLS_UPPER(TRIM(wcodi));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_CONTRACTE_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi contracte no existent o donat de baixa logica';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure MODIFICACTRALPRODCTRALDISTR
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MODIFICACTRALPRODCTRALDISTR" (wcentralprod   IN  CENTRALPROD_CENTRALDISTR.centralprod%TYPE,
                                      wlinia         IN  CENTRALPROD_CENTRALDISTR.linia%TYPE,
                                      wcentraLdistr  IN  CENTRALPROD_CENTRALDISTR.centraldistr%TYPE,
                                      wactivat       IN  CENTRALPROD_CENTRALDISTR.activat%TYPE,
                                      wusuari        IN  CENTRALPROD_CENTRALDISTR.usuarialta%TYPE,
                                      RTN            OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             modificaCtralProdCtralDistr
FUNCIO:          activa o desactiva la connexio linia/central prod. de una central de distribuci (nomes 1 activa)
PARMETRES  IN:  centralprod: Codi de la central de produccio
                 linia:  codi de la linia de comunicaci
                 centraldistr: Codi de la central de distribuci
                 activat: connexio activa ('S', 'N'), una central de distribuci pot tenir nomes una connexio
                          linia-central producci activa
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_CENTRALPRDLIN_NO_EXISTENT EXCEPTION;
  CODI_CONNCTRALDISTR_JA_ACTIVA EXCEPTION;
  CODI_CONNCTRALDISTR_NO_ACTIVA EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'modificaCtralProdCtralDistr';
    xdades_ent := 'Central Prod.: '    || wcentralprod  ||
                  '. Linia com.: '     || wlinia        ||
                  '. Central Distr.: ' || wcentraldistr ||
                  '. Activat: '        || wactivat      ||
                  '. Usuari: '         || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

  IF (wcentralprod IS NULL) OR (wlinia IS NULL) OR 
     (wcentraldistr IS NULL) OR (wactivat IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio relacio central/linia/distr existent

   SELECT COUNT(*) 
    INTO wcount
     FROM CENTRALPROD_CENTRALDISTR
    WHERE centralprod = NLS_UPPER(TRIM(wcentralprod))
    AND linia = NLS_UPPER(TRIM(wlinia))
    AND centraldistr = NLS_UPPER(TRIM(wcentraldistr));
     
   IF (wcount = 0) THEN
      RAISE CODI_CENTRALPRDLIN_NO_EXISTENT;
   END IF;

-- Validacio una central distribuci te nomes una connexio central/linia activa

   IF (wactivat = 'S') THEN
       SELECT COUNT(*) 
        INTO wcount
         FROM CENTRALPROD_CENTRALDISTR
        WHERE centraldistr = NLS_UPPER(TRIM(wcentraldistr))
        AND activat = 'S' ;
     
       IF (wcount > 0) THEN
          RAISE CODI_CONNCTRALDISTR_JA_ACTIVA;
       END IF;
   END IF;
   
-- Actualitzaci de les dades
 
   UPDATE CENTRALPROD_CENTRALDISTR 
     SET activat = NLS_UPPER(TRIM(wactivat)),
         datamod      = SYSDATE,
         usuarimod    = NLS_UPPER(TRIM(wusuari))
    WHERE centralprod  = NLS_UPPER(TRIM(wcentralprod))
    AND linia = NLS_UPPER(TRIM(wlinia))
    AND centraldistr = NLS_UPPER(TRIM(wcentraldistr));
    
-- Validacio central distribuci te al menys una connexi activada, es un warning
    SELECT COUNT(*) 
      INTO wcount
       FROM CENTRALPROD_CENTRALDISTR
      WHERE centraldistr = NLS_UPPER(TRIM(wcentraldistr))
      AND activat = 'S';
    
     IF (wcount = 0) THEN
        RAISE CODI_CONNCTRALDISTR_NO_ACTIVA;
     END IF;
 
    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

        WHEN  CODI_CENTRALPRDLIN_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Connexio Central Prod/Linia com./Central distr. no existeix o ja donada de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
       
        WHEN  CODI_CONNCTRALDISTR_JA_ACTIVA  THEN
          xid_error := -003;
          xrtn  := 'KO';
          xdades_sort := 'Central Distr. te ja una connexi activa a una central de producci';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
 
        WHEN  CODI_CONNCTRALDISTR_NO_ACTIVA  THEN
          xid_error := -004;
          xrtn  := 'OK';
          xdades_sort := 'Warning: Central Distr. no te una connexi activa a una central de producci';
          RTN         := 'WARNING.' || xdades_sort;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure MODIFICALINIACOM
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MODIFICALINIACOM" (wcodi       IN  LINIA_COM.codi%TYPE,
                           wcapacitat  IN  LINIA_COM.capacitatmax%TYPE,
                           wusuari     IN  LINIA_COM.usuarialta%TYPE,
                           RTN      OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             modificaLiniaCom
FUNCIO:          Modifica de les linies de comunicaci (nomes la capacitat).
PARMETRES  IN:  codi: Codi de la linia
                 capacitatmax:  capacitat maxima de transport de la linia
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_LINIA_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'modificaLiniaCom';
    xdades_ent := 'Codi: '        || wcodi  ||
                  '. Capacitat: ' || wcapacitat   ||
                  '. Usuari: '    || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wcapacitat IS NULL) or (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio codi LINIA NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM LINIA_COM
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_LINIA_NO_EXISTENT;
   END IF;

-- Actualitzaci de les dades
 
    UPDATE LINIA_COM 
      SET capacitatmax = wcapacitat,
          datamod      = SYSDATE,
          usuarimod    = NLS_UPPER(TRIM(wusuari))
      WHERE codi = NLS_UPPER(TRIM(wcodi));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
          
         WHEN  CODI_LINIA_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi linia de comunicaci no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
        
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure MODIFICAMODEL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MODIFICAMODEL" (wcodi     IN  MODELCMPT.codi%TYPE,
                        wpotencia IN  MODELCMPT.potenciamax%TYPE,
                        wempresa  IN  MODELCMPT.empresa%TYPE,
                        wanyfab   IN  MODELCMPT.anyfab%TYPE,
                        wusuari   IN  MODELCMPT.usuarialta%TYPE,
                        RTN      OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             modificaModel
FUNCIO:          modifica dels models dels comptadors (a excepcio del codi.
PARMETRES  IN:  codi: Codi del model
                 potenciamax:  potencia maxima 
                 empresa:  empresa de fabricacio
                 anyfab:   any de fabricaci
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
    
-- Dades de treball per al procediment
  wcount       NUMBER;

-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_MODEL_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'modificaModel';
    xdades_ent := 'Codi: '        || wcodi     ||
                  '. Potencia: '  || wpotencia ||  
                  '. Empresa: '   || wempresa  ||
                  '. Any fab.: '  || wanyfab   ||  
                  '. Usuari: '    || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wpotencia IS NULL) OR (wempresa IS NULL) OR (wanyfab IS NULL) OR (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio codi MODELCMPT NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM MODELCMPT
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
     
   IF (wcount = 0) THEN
      RAISE CODI_MODEL_NO_EXISTENT;
   END IF;

-- Actualitzaci de les dades
 
    UPDATE MODELCMPT 
    SET potenciamax = wpotencia,  
        empresa = NLS_UPPER(TRIM(wempresa)),
        anyfab = wanyfab,
        datamod = SYSDATE,
        usuarimod = NLS_UPPER(TRIM(wusuari))
   WHERE codi     = NLS_UPPER(TRIM(wcodi));
 
    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
        WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

         WHEN  CODI_MODEL_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi model no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
--------------------------------------------------------
--  DDL for Procedure MODIFICAPAIS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MODIFICAPAIS" (wcodi     IN  PAIS.codi%TYPE,
                       wnom      IN  PAIS.nom%TYPE,
                       wusuari   IN  PAIS.usuarimod%TYPE,
                       RTN      OUT  NOCOPY VARCHAR2) 
  IS
  
/**************************************************************************************
NOM:             modificaPais
FUNCIO:          Modificaci dels paisos (nomes el nom).
PARMETRES  IN:  codi: Codi ISO del pais
                 nom:  nom del pais
                 usuari: usuari que executa el procediment
            OUT: RTN: Valor 'OK' o 'ERROR'+Descripcio de l'error
**************************************************************************************/

-- Dades de treball per al log
  xModul        CHAR(30);
  xdades_ent    VARCHAR2(1024);
  xdades_sort   VARCHAR2(1024);
  xrtn          CHAR(2);
  xid_error     NUMBER;
   
-- Dades de treball per al procediment
  wcount       NUMBER;
 
-- Excepcions  
  VALORS_CAMPS_ARE_NULL EXCEPTION;
  CODI_PAIS_NO_EXISTENT EXCEPTION;
  
  
BEGIN

-- Es preparen les dades per al log

    xModul := 'modificaPais';
    xdades_ent := 'Codi: '      || wcodi ||
                  '. Nom: '     || wnom  ||
                  '. Usuari: '  || wusuari ; 
    xdades_sort := '';
    wcount  := 0;

  
-- Validacio dades d'entrada. Valors no null

   IF (wcodi IS NULL) OR (wnom IS NULL) or (wusuari IS NULL) THEN
       RAISE VALORS_CAMPS_ARE_NULL;
   END IF;
 
-- Validacio codi PAIS NO existent

   SELECT COUNT(*) 
    INTO wcount
     FROM PAIS
    WHERE codi = NLS_UPPER(TRIM(wcodi))
    AND databaixa IS NULL;
 
    
   IF (wcount = 0) THEN
      RAISE CODI_PAIS_NO_EXISTENT;
   END IF;


-- Actualitzaci de les dades

    UPDATE PAIS
       SET nom       = NLS_UPPER(TRIM(wnom)),
           datamod   = SYSDATE,
           usuarimod = NLS_UPPER(TRIM(wusuari))
     WHERE codi      = NLS_UPPER(TRIM(wcodi));

    xrtn  := 'OK';
    RTN   := 'OK';
    xid_error := 0;

    insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);

    
    EXCEPTION  
         WHEN  VALORS_CAMPS_ARE_NULL THEN
          xid_error := -001;
          xrtn  := 'KO';
          xdades_sort := 'Un o mes parametres d''entrada tenen valor nul';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
 
         WHEN  CODI_PAIS_NO_EXISTENT  THEN
          xid_error := -002;
          xrtn  := 'KO';
          xdades_sort := 'Codi pais no existent o donat de baixa';
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul,xdades_ent, xdades_sort, xrtn, xid_error, wusuari);
       
        WHEN OTHERS THEN
          xid_error := -999;
          xrtn  := 'KO';
          xdades_sort := SUBSTR (SQLERRM, 1, 100);
          RTN         := 'ERROR.' || xdades_sort;
          ROLLBACK;
          insertLog(xModul, xdades_ent, xdades_sort ,xrtn, xid_error, wusuari);
        
END;

/

/
