--------------------------------------------------------------------------------
-- CREACIO DE LES TAULES PRINCIPALS
--------------------------------------------------------------------------------

--------------------------------------------------------
--  1. Taula de Paisos
--------------------------------------------------------

  CREATE TABLE PAIS
   (CODI CHAR(2 CHAR) CONSTRAINT PK_PAIS PRIMARY KEY, 
	NOM VARCHAR2(50 CHAR) CONSTRAINT NN_PAIS_nom NOT NULL, 
  DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE));
  
--------------------------------------------------------
-- 2. Taula de Models de comptadors
--------------------------------------------------------

  CREATE TABLE MODELCMPT
   (CODI CHAR(12 BYTE) CONSTRAINT PK_MODELCMPT PRIMARY KEY, 
	POTENCIAMAX NUMBER CONSTRAINT NN_MODELCMPT_potenciamax NOT NULL,
	EMPRESA VARCHAR2(50 CHAR) CONSTRAINT NN_MODELCMPT_empresa NOT NULL,
	ANYFAB NUMBER CONSTRAINT NN_MODELCMPT_any NOT NULL, 
    DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE));

--------------------------------------------------------
-- 3. Taula de Comptadors
--------------------------------------------------------

  CREATE TABLE COMPTADOR
   (CODI VARCHAR2(12 BYTE) CONSTRAINT PK_COMPTADOR PRIMARY KEY, 
	IDMODEL CHAR(12 BYTE) CONSTRAINT NN_COMPTADOR_model NOT NULL
                        CONSTRAINT FK1_COMPTADOR_model REFERENCES MODELCMPT(codi),  
	ADRECA VARCHAR2(100 BYTE) CONSTRAINT NN_COMPTADOR_adreca NOT NULL,
	POBLACIO VARCHAR2(50 BYTE) CONSTRAINT NN_COMPTADOR_poblacio NOT NULL,
 	CODIPOSTAL VARCHAR2(50 BYTE) CONSTRAINT NN_COMPTADOR_codipostal NOT NULL,
	PAIS CHAR(2 CHAR) CONSTRAINT NN_COMPTADOR_pais NOT NULL
                    CONSTRAINT FK2_COMPTADOR_pais REFERENCES PAIS(codi),   
	CONSUM NUMBER, 
    LECTURA NUMBER,
	DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE));

--------------------------------------------------------
-- 4. Taula de Clients
--------------------------------------------------------

  CREATE TABLE CLIENT
   (CODI VARCHAR2(12 BYTE) CONSTRAINT PK_CLIENT PRIMARY KEY,
	TIPUS CHAR(1 BYTE) CONSTRAINT NN_CLIENT_tipus NOT NULL 
                     CONSTRAINT CH_CLIENT_tipus CHECK (tipus IN ('F', 'J')),
	NOM VARCHAR2(50 BYTE) CONSTRAINT NN_CLIENT_nom NOT NULL, 
	COGNOM1 VARCHAR2(30 BYTE),
	COGNOM2 VARCHAR2(30 BYTE), 
	ADRECA VARCHAR2(100 BYTE) CONSTRAINT NN_CLIENT_adreca NOT NULL,
	POBLACIO VARCHAR2(50 BYTE) CONSTRAINT NN_CLIENT_poblacio NOT NULL, 
	CODIPOSTAL VARCHAR2(20 BYTE) CONSTRAINT NN_CLIENT_codipostal NOT NULL,
	PAIS CHAR(2 CHAR) CONSTRAINT NN_CLIENT_pais NOT NULL
                      CONSTRAINT FK1_CLIENT_pais REFERENCES PAIS(codi),  
	TELEFON CHAR(20 BYTE), 
	EMAIL CHAR(30 BYTE), 
	DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE));
 

--------------------------------------------------------
-- 5. Taula de Contractes
--------------------------------------------------------

  CREATE TABLE CONTRACTE
   (CODI VARCHAR2(12 BYTE) CONSTRAINT PK_CONTRACTE PRIMARY KEY,
	DATAINICI DATE CONSTRAINT NN_CONTRACTE_datainici NOT NULL,
	DATAFI DATE, 
	COMPTADOR VARCHAR2(12 BYTE) CONSTRAINT NN_CONTRACTE_comptador NOT NULL
                                CONSTRAINT FK1_CONTRACTE_comptador REFERENCES COMPTADOR(codi),  
	CLIENT VARCHAR2(12 BYTE) CONSTRAINT NN_CONTRACTE_client NOT NULL
                             CONSTRAINT FK2_CONTRACTE_client REFERENCES CLIENT(codi),
	POTENCIA NUMBER CONSTRAINT NN_CONTRACTE_potencia NOT NULL 
                    CONSTRAINT CH_CONTRACTE_potencia CHECK (potencia > 0),
	PREUKWH NUMBER CONSTRAINT NN_CONTRACTE_preukWh NOT NULL 
                   CONSTRAINT CH_CONTRACTE_preukWh CHECK (preukWh > 0),
	DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE));


--------------------------------------------------------
-- 6. Taula de Linies de comunicaci
--------------------------------------------------------

  CREATE TABLE LINIA_COM
   (CODI VARCHAR2(12 BYTE) CONSTRAINT PK_LINIA_COM PRIMARY KEY,
	CAPACITATMAX NUMBER CONSTRAINT NN_LINIA_COM_capacitatmax NOT NULL 
                        CONSTRAINT CH_LINIA_COM_capacitatmax CHECK (capacitatmax > 0),
	DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE));

-------------------------------------------------------
-- 7. Taula Centrals de distribuci 
--------------------------------------------------------
  CREATE TABLE CENTRAL_DISTR
   (	CODI VARCHAR2(12 BYTE) CONSTRAINT PK_CENTRAL_DISTR PRIMARY KEY,
	ADRECA VARCHAR2(100 BYTE) CONSTRAINT NN_CENTRAL_DISTR_adreca NOT NULL,
	POBLACIO VARCHAR2(50 BYTE) CONSTRAINT NN_CENTRAL_DISTR_poblacio NOT NULL,
	CODIPOSTAL VARCHAR2(20 BYTE) CONSTRAINT NN_CENTRAL_DISTR_codipostal NOT NULL,
	PAIS CHAR(2 CHAR) CONSTRAINT NN_CENTRAL_DISTR_pais NOT NULL
                    CONSTRAINT FK1_CENTRAL_DISTR_pais REFERENCES PAIS(codi), 
	ENERGIAMAX NUMBER CONSTRAINT NN_CENTRAL_DISTR_energiamax NOT NULL 
                         CONSTRAINT CH_CENTRAL_DISTR_energiamax CHECK (energiamax > 0),
	DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE));
    
--------------------------------------------------------  
-- 8. Taula Centrals de produccio
--------------------------------------------------------

  CREATE TABLE CENTRAL_PROD
   (CODI VARCHAR2(12 BYTE) CONSTRAINT PK_CENTRAL_PROD PRIMARY KEY,
	ADRECA VARCHAR2(100 BYTE) CONSTRAINT NN_CENTRAL_PROD_adreca NOT NULL,
	POBLACIO VARCHAR2(50 BYTE) CONSTRAINT NN_CENTRAL_PROD_poblacio NOT NULL,
	CODIPOSTAL VARCHAR2(20 BYTE) CONSTRAINT NN_CENTRAL_PROD_codipostal NOT NULL,
	PAIS CHAR(2 CHAR) CONSTRAINT NN_CENTRAL_PROD_pais NOT NULL
                      CONSTRAINT FK1_CENTRAL_PROD_pais REFERENCES PAIS(codi),   
	TIPUS CHAR(1 BYTE) CONSTRAINT NN_CENTRAL_PROD_tipus NOT NULL
                     CONSTRAINT CH_CENTRAL_PROD_tipus CHECK (tipus IN ('S', 'T', 'C','E','N' )),  
	NUMMOLINS NUMBER, 
	NUMPANELS NUMBER, 
	KGCO2 NUMBER, 
	KGREBUIG NUMBER, 
	ENERGIAMIN NUMBER, 
	DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE));

--------------------------------------------------------
--  9. Taula de connexins Comptadors - Centrals de distribuci
--------------------------------------------------------

  CREATE TABLE COMPTADOR_CENTRALDISTR
   (COMPTADOR VARCHAR2(12 BYTE) CONSTRAINT FK1_COMPTADOR_CENTRALDISTR REFERENCES COMPTADOR(CODI),
	CENTRALDISTR VARCHAR2(12 BYTE) CONSTRAINT FK2_COMPTADOR_CENTRALDISTR REFERENCES CENTRAL_DISTR(CODI), 
	ACTIVAT CHAR(1 BYTE) CONSTRAINT NN_COMPTADORCTRALDISTR_activat NOT NULL 
                         CONSTRAINT CH_COMPTADORCTRALDISTR_activat CHECK (activat IN ('S', 'N')),
	DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE),
  CONSTRAINT PK_COMPTADOR_CENTRALDISTR PRIMARY KEY (COMPTADOR,CENTRALDISTR));
  
  
--------------------------------------------------------
-- 10. Taula connexios Central de producio - Centrals de distribuci
--------------------------------------------------------

  CREATE TABLE CENTRALPROD_CENTRALDISTR
   (CENTRALPROD VARCHAR2(12 BYTE) CONSTRAINT FK1_CENTRALPROD_CENTRALDISTR REFERENCES CENTRAL_PROD(CODI), 
	LINIA VARCHAR2(12 BYTE) CONSTRAINT FK2_CENTRALPROD_CENTRALDISTR REFERENCES LINIA_COM(CODI), 
    CENTRALDISTR VARCHAR2(12 BYTE) CONSTRAINT FK3_CENTRALPROD_CENTRALDISTR REFERENCES CENTRAL_DISTR(CODI),  
    ACTIVAT CHAR(1 BYTE) CONSTRAINT CH_CTRALPRODCTRALDISTR_ACTIVAT CHECK (activat IN ('S', 'N')),
	DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE),
  CONSTRAINT PK_CENTRALPROD_CENTRALDISTR PRIMARY KEY (CENTRALPROD,LINIA,CENTRALDISTR));
	

--------------------------------------------------------
--  11. Taula de Lectures dels comptadors 
--------------------------------------------------------

  CREATE TABLE LECTURA
   (DATALECTURA DATE CONSTRAINT NN_LECTURA_datalectura NOT NULL,
	COMPTADOR VARCHAR2(12 BYTE) CONSTRAINT NN_LECTURA_comptador NOT NULL
                                CONSTRAINT FK1_LECTURA_comptador REFERENCES COMPTADOR(codi),  
	CENTRALDISTR VARCHAR2(12 BYTE) CONSTRAINT NN_LECTURA_centraldistr NOT NULL
                                 CONSTRAINT FK1_LECTURA_centraldistr REFERENCES CENTRAL_DISTR(codi),  
	LINIACOM VARCHAR2(12 BYTE) CONSTRAINT NN_LECTURA_liniacom NOT NULL
                             CONSTRAINT FK2_LECTURA_liniacom REFERENCES LINIA_COM(codi),   
	CENTRALPROD VARCHAR2(12 BYTE) CONSTRAINT NN_LECTURA_centralprod NOT NULL
                                CONSTRAINT FK1_LECTURA_centralprod REFERENCES CENTRAL_PROD(codi),
	VALOR NUMBER CONSTRAINT NN_LECTURA_valor NOT NULL 
                 CONSTRAINT CH_LECTURA_valor CHECK (valor >= 0), 
	CONSUM NUMBER CONSTRAINT NN_LECTURA_consum NOT NULL 
                  CONSTRAINT CH_LECTURA_consum CHECK (consum >= 0), 
	TIPUS CHAR(1 BYTE) CONSTRAINT NN_LECTURA_tipus NOT NULL 
                     CONSTRAINT CH_LECTURA_tipus CHECK (tipus IN ('P', 'T')),
	DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE));  


--------------------------------------------------------------------------------
-- CREACIO DE LES TAULES DEL HISTORIC PEL MODUL ESTADISTIC
--------------------------------------------------------------------------------

--------------------------------------------------------
--  Taula del Historic de Centrals de producci
--------------------------------------------------------

  CREATE TABLE HCENTRAL_PROD
   (CODI VARCHAR2(12 BYTE),
	ANYCONSUM NUMBER(4,0),
	CONSUM NUMBER CONSTRAINT NN_HCENTRAL_PROD_consum NOT NULL 
                  CONSTRAINT CH_HCENTRAL_PROD_consum CHECK (consum >= 0),
	DATAALTA DATE, 
	DATABAIXA DATE, 
	DATAMOD DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE),
  CONSTRAINT PK_HCENTRAL_PROD PRIMARY KEY (CODI, ANYCONSUM));
  
--------------------------------------------------------
--  Taula del Historic de Comptadors
--------------------------------------------------------

  CREATE TABLE HCOMPTADOR
   (CODI VARCHAR2(12 BYTE) CONSTRAINT PK_HCOMPTADOR PRIMARY KEY, 
	CONSUM NUMBER CONSTRAINT NN_HCOMPTADOR_consum NOT NULL 
                  CONSTRAINT CH_HCOMPTADOR_consum CHECK (consum >= 0), 
	DATAALTA DATE, 
    DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE));

--------------------------------------------------------
--  Taula del Historic de Linies de comunicaci
--------------------------------------------------------

  CREATE TABLE HLINIA_COM
   (CODI VARCHAR2(12 BYTE), 
	ANYCONSUM NUMBER(4,0),
	CONSUM NUMBER CONSTRAINT NN_HLINIA_COM_consum NOT NULL 
                     CONSTRAINT CH_HLINIA_COM_consum CHECK (consum >= 0),
    VALORMIGCONSUM  NUMBER CONSTRAINT NN_HLINIA_COM_valormigconsum NOT NULL 
                    CONSTRAINT CH_HLINIA_COM_valormigconsum CHECK (valormigconsum >= 0),
	CAPACITATMAX  NUMBER CONSTRAINT NN_HLINIA_COM_capacitatmax NOT NULL 
	              CONSTRAINT CH_HLINIA_COM_capacitatmax CHECK (capacitatmax >= 0),
	DATAALTA DATE, 
    DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE),
  CONSTRAINT PK_HLINIA_COM PRIMARY KEY (CODI, ANYCONSUM));
 

--------------------------------------------------------
--  Taula del Historic de Clients
--------------------------------------------------------
  
  CREATE TABLE HCLIENT
   (ANYCONSUM NUMBER(4,0), 
	NUMCLIENTS NUMBER,
	CONSUMTOTAL NUMBER CONSTRAINT NN_HCLIENT_consumtotal NOT NULL 
                        CONSTRAINT CH_HCLIENT_consumtotal CHECK (consumtotal >= 0), 
	CONSUMMIG NUMBER CONSTRAINT NN_HCLIENT_consummig NOT NULL 
                        CONSTRAINT CH_HCLIENT_consummig CHECK (consummig >= 0), 
	DATAALTA DATE, 
	DATAMOD DATE, 
	DATABAIXA DATE, 
	USUARIALTA VARCHAR2(15 BYTE), 
	USUARIMOD VARCHAR2(15 BYTE), 
	USUARIBAIXA VARCHAR2(15 BYTE),
  CONSTRAINT HCLIENT PRIMARY KEY (ANYCONSUM));

--------------------------------------------------------------------------------
-- CREACIO DE LA TAULA DE LOG
--------------------------------------------------------------------------------

  CREATE TABLE LOG_DB 
   (MODUL VARCHAR2(35 BYTE), 
	DADESIN VARCHAR2(1024 BYTE), 
	DADESOUT VARCHAR2(1024 BYTE), 
	RESULTAT CHAR(2 BYTE), 
	IDERROR NUMBER, 
	USUARI VARCHAR2(15 BYTE), 
	DATALOG TIMESTAMP (6) );
